@@ -103,12 +103,13 @@ Ephy.getWebAppIcon = function(baseURL)
 
 Ephy.PreFillUserMenu = class PreFillUserMenu
 {
-    constructor(manager, userElement, users, passwordElement)
+    constructor(manager, formAuth, users)
     {
         this._manager = manager;
-        this._userElement = userElement;
+        this._formAuth = formAuth;
+        this._userElement = formAuth.usernameNode;
         this._users = users;
-        this._passwordElement = passwordElement;
+        this._passwordElement = formAuth.passwordNode;
         this._selected = null;
         this._wasEdited = false;
 
@@ -165,7 +166,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
         if (newSelect) {
             this._selected = newSelect;
             this._userElement.value = this._selected.firstElementChild.textContent;
-            this._manager.preFill();
+            this._usernameSelected();
         } else {
             this._passwordElement.value = '';
         }
@@ -186,13 +187,14 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
             'z-index: 2147483647;' +
             'cursor: default;' +
             'background-color: white;' +
-            'box-shadow: 5px 5px 5px black;' +
+            'box-shadow: 5px 5px 5px rgba(0,0,0,0.2);' +
             'border-top: 0px;' +
             'border-radius: 8px;' +
+            'padding: 12px 0px;' +
             '-webkit-user-modify: read-only ! important;';
-        mainDiv.style.width = this._userElement.offsetWidth;
-        mainDiv.style.left = elementRect.left + document.body.scrollLeft;
-        mainDiv.style.top = elementRect.top + document.body.scrollTop;
+        mainDiv.style.width = this._userElement.offsetWidth + 'px';
+        mainDiv.style.left = elementRect.left + document.body.scrollLeft + 'px';
+        mainDiv.style.top = elementRect.top + elementRect.height + document.body.scrollTop + 'px';
 
         let ul = document.createElement('ul');
         ul.style.cssText = 'margin: 0; padding: 0;';
@@ -209,6 +211,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
             li.style.cssText = 'list-style-type: none ! important;' +
                 'background-image: none ! important;' +
                 'padding: 3px 6px ! important;' +
+                'color: black;' +
                 'margin: 0px;';
             // FIXME: selection colors.
             li.tabindex = -1;
@@ -221,6 +224,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
             anchor.style.cssText = 'font-weight: normal ! important;' +
                 'font-family: sans ! important;' +
                 'text-decoration: none ! important;' +
+                'color: black;' +
                 '-webkit-user-modify: read-only ! important;';
             // FIXME: selection colors.
             anchor.textContent = user;
@@ -230,7 +234,7 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
                 this._userElement.value = user;
                 this._selected = li;
                 this._removeMenu();
-                this._manager.preFill();
+                this._usernameSelected();
             }, true);
         }
 
@@ -246,6 +250,13 @@ Ephy.PreFillUserMenu = class PreFillUserMenu
         if (menu)
             menu.parentNode.removeChild(menu);
     }
+
+    _usernameSelected()
+    {
+        this._formAuth.username = this._userElement.value;
+        this._passwordElement.value = '';
+        this._manager.preFill(this._formAuth);
+    }
 }
 
 Ephy.formControlsAssociated = function(pageID, frameID, forms, serializer)
@@ -330,11 +341,11 @@ Ephy.PasswordManager = class PasswordManager
 
     _onQueryResponse(username, password, id)
     {
+        Ephy.log(`Received password query response for username=${username}`);
+
         let element = this._takePendingPromise(id);
         if (element) {
-            if (username === '')
-                username = null;
-            if (password !== '')
+            if (password)
                 element.resolver({username, password});
             else
                 element.resolver(null);
@@ -343,6 +354,8 @@ Ephy.PasswordManager = class PasswordManager
 
     query(origin, targetOrigin, username, usernameField, passwordField)
     {
+        Ephy.log(`Querying passwords for origin=${origin}, targetOrigin=${targetOrigin}, username=${username}, usernameField=${usernameField}, passwordField=${passwordField}`);
+
         return new Promise((resolver, reject) => {
             let promiseID = this._promiseCounter++;
             Ephy.queryPassword(origin, targetOrigin, username, usernameField, passwordField, promiseID, this._pageID, this._frameID);
@@ -352,6 +365,8 @@ Ephy.PasswordManager = class PasswordManager
 
     save(origin, targetOrigin, username, password, usernameField, passwordField, isNew)
     {
+        Ephy.log(`Saving password for origin=${origin}, targetOrigin=${targetOrigin}, username=${username}, usernameField=${usernameField}, passwordField=${passwordField}, isNew=${isNew}`);
+
         window.webkit.messageHandlers.passwordManagerSave.postMessage({
             origin, targetOrigin, username, password, usernameField, passwordField, isNew,
             pageID: this._pageID
@@ -361,6 +376,8 @@ Ephy.PasswordManager = class PasswordManager
     // FIXME: Why is pageID a parameter here?
     requestSave(origin, targetOrigin, username, password, usernameField, passwordField, isNew, pageID)
     {
+        Ephy.log(`Requesting to save password for origin=${origin}, targetOrigin=${targetOrigin}, username=${username}, usernameField=${usernameField}, passwordField=${passwordField}, isNew=${isNew}`);
+
         window.webkit.messageHandlers.passwordManagerRequestSave.postMessage({
             origin, targetOrigin, username, password, usernameField, passwordField, isNew,
             pageID
@@ -369,6 +386,8 @@ Ephy.PasswordManager = class PasswordManager
 
     _onQueryUsernamesResponse(users, id)
     {
+        Ephy.log(`Received query usernames response with users=${users}`);
+
         let element = this._takePendingPromise(id);
         if (element)
             element.resolver(users);
@@ -376,6 +395,8 @@ Ephy.PasswordManager = class PasswordManager
 
     queryUsernames(origin)
     {
+        Ephy.log(`Requesting usernames for origin=${origin}`);
+
         return new Promise((resolver, reject) => {
             let promiseID = this._promiseCounter++;
             Ephy.queryUsernames(origin, promiseID, this._pageID, this._frameID);
@@ -440,7 +461,7 @@ Ephy.FormManager = class FormManager
             Ephy.passwordManager.queryUsernames(formAuth.origin).then(users => {
                 if (users.length > 1) {
                     Ephy.log('More than one saved username, hooking menu for choosing which one to select');
-                    this._preFillUserMenu = new Ephy.PreFillUserMenu(this, formAuth.usernameNode, users, formAuth.passwordNode);
+                    this._preFillUserMenu = new Ephy.PreFillUserMenu(this, formAuth, users);
                 }
                 this.preFill(formAuth);
             });
@@ -460,7 +481,7 @@ Ephy.FormManager = class FormManager
             formAuth.username,
             formAuth.usernameField,
             formAuth.passwordField).then(authInfo => {
-                if (!authInfo.username) {
+                if (!authInfo) {
                     Ephy.log('No result');
                     return;
                 }
@@ -506,7 +527,7 @@ Ephy.FormManager = class FormManager
             formAuth.username,
             formAuth.usernameField,
             formAuth.passwordField).then(authInfo => {
-                if (authInfo.username) {
+                if (authInfo) {
                     if (authInfo.username == formAuth.username && authInfo.password == formAuth.password) {
                         Ephy.log('User/password already stored. Not asking about storing.');
                         return;
@@ -535,7 +556,7 @@ Ephy.FormManager = class FormManager
                                                  formAuth.password,
                                                  formAuth.usernameField,
                                                  formAuth.passwordField,
-                                                 authInfo.username == null,
+                                                 authInfo == null,
                                                  this._pageID);
             }
         );
